#!/bin/sh
#rpxqimage

R1CM=3
R2D=4
R1CL=5
R2CM=6
R3=7
R3D=8
R3L=9
R3P=10
P01=11
R3A=12
R3G=13
R4=14
R4C=15
D01=16
R4A=17
R4CM=18
R4AC=19
R3GV2=20
R2600=21
R2100=22
RM2100=23
R3600=24
R1350=25
R2200=26
R2350=27
IR1200G=28
R1800=29
R2100D=30
RA67=31
RA69=32
RA71=33
CR6006=34
CR6008=35
CR6009=36
RA70=37
RA75=38
RA72=39

#$1 - file; $2 - value; $3 - size; $4 - offset
write_value() {
    printf $(eval printf "$(printf '\\\\x%%02x%.0s' $(seq 1 $3))" $(printf '$(($2>>%d&0xff)) ' $(seq 0 8 $(($3*8-8))))) | \
	dd of="$1" bs=1 count=$3 seek=${4:-0} conv=notrunc 2>/dev/null
}

output=
signature="/dev/zero"
files=""
state="opt"
model=
hdr_size=

for arg in "$@"
do
	case "$state" in
	"opt")
		state="$arg"
		continue;
		;;
	"-f")
		files="$files $arg"
		;;
	"-o")
		output="$arg"
		;;
	"-s")
		signature="$arg"
		;;
	"-m")
		model="$(eval printf \$$arg)"
		;;
	"-M")
		model="$(($arg))"
		;;
	"-H")
		hdr_size="$(($arg))"
		;;
	*)
		echo "Invalid option: $arg"
		exit 1
		;;
	esac
	state="opt"
done

[ $(($hdr_size)) -lt 48 ] && hdr_size=48
hdr_size=$((($hdr_size / 4) * 4))

model="${model:-$(($(eval printf \$$(cat /proc/xiaoqiang/model))))}" 2>/dev/null
[ -z "$output" ] && { echo "-o required"; exit 1; }
[ "$signature" != "/dev/zero" ] && sig_size=$(stat -c%s "$signature") || sig_size=$((0x110))

set -e

printf "HDR1" > "$output"
dd if=/dev/zero bs=$((hdr_size - 4)) count=1 2>/dev/null >> "$output"
write_value "$output" "$model" 2 14

seg_offset=16
cur_offset=$hdr_size

for segment in $(echo $files)
do
	seg_name="$(basename $segment)"
	write_value "$output" $cur_offset 4 $seg_offset
	seg_offset=$(($seg_offset+4))
	printf "\xbe\xba\x00\x00\xff\xff\xff\xff\x00\x00\x00\x00\xff\xff\x00\x00" >> "$output"
	printf "$seg_name" >> "$output"
	dd if=/dev/zero bs=1 count=$((32-${#seg_name})) 2>/dev/null >> "$output"
	dd if="$segment" 2>/dev/null >> "$output"
	write_value "$output" $(stat -c%s "$segment") 4 $(($cur_offset+8))
	cur_offset=$(stat -c%s "$output")
	[ $(($cur_offset%4)) -eq 0 ] || { dd if=/dev/zero bs=1 count=$((4-$cur_offset%4)) 2>/dev/null >> "$output"; }
	cur_offset=$(stat -c%s "$output")
done

write_value "$output" $cur_offset 4 4
dd if="$signature" bs=1 count=$sig_size 2>/dev/null >> "$output"
[ "$signature" = "/dev/zero" ] && write_value "$output" $((0x100)) 4 $cur_offset
write_value "$output" $(($(dd if="$output" skip=1 bs=12 2>/dev/null | ubicrc32))) 4 8

mkxqimage -v "$output"
