#!/bin/sh
#rpxqimage

set -e

if [ "`cat "$1" | head -c4`" != "UBI#" ]
then
	exit 1
fi

tmp_dir="$(printf $(mktemp -ud /tmp/$(basename $0).$(basename $1).XXXXXX | sed -r 's/\.+/_/g'))"

on_exit() {
	set +e
	umount "$tmp_dir/mnt" 2>/dev/null
	umount "$tmp_dir" 2>/dev/null
	rm -rf "$tmp_dir" 2>/dev/null || {
		umount "$tmp_dir" 2>/dev/null
		rm -rf "$tmp_dir" 2>/dev/null
	}
}
trap on_exit EXIT

mkdir "$tmp_dir"
mount -t tmpfs tmpfs "$tmp_dir"
ubireader_extract_images.lua -w -o "$tmp_dir" "$1"

kernel="$tmp_dir/$(ls $tmp_dir | grep kernel)"
rootfs="$tmp_dir/$(ls $tmp_dir | grep rootfs)"

ROOTFS_SIG=`hexdump -n 4 -e '"%_p"' "$rootfs"`
[ "$ROOTFS_SIG" = "hsqs" ] || { echo "rootfs is not squashfs."; exit 1; }

KERNEL_SIG=`hexdump -n 4 -e '1/1 "%02x"' "$kernel"`
[ "$KERNEL_SIG" = "d00dfeed" ] || { echo "invalid kernel img"; exit 1; }

mkdir "$tmp_dir/mnt"
squashfuse "$rootfs" "$tmp_dir/mnt"

if [ "$2" = "-V" ]
then
	cat "$tmp_dir/mnt/usr/share/xiaoqiang/xiaoqiang_version"
	exit 0
fi

if [ "$(uci -q get $tmp_dir/mnt/usr/share/xiaoqiang/xiaoqiang_version.version.HARDWARE)" != "$(uci -q get /usr/share/xiaoqiang/xiaoqiang_version.version.HARDWARE)" ]
then
	exit 1
fi

if [ "$2" = "-b" ]
then
	cp "$tmp_dir/mnt/usr/share/xiaoqiang/xiaoqiang_version" "$tmp_dir/xiaoqiang_version"
	rm -f "$rootfs"
	rm -f "$kernel"
	mv -f "$1" "$tmp_dir/root.ubi"
	mkxqimage.mkimg.sh -f "$tmp_dir/xiaoqiang_version" -f "$tmp_dir/root.ubi" -o "$1"
fi

exit 0
