#!/bin/sh
#rpxqimage

set -e

state="opt"
kernel=
rootfs=
output=
data=
image_seq=

for arg in "$@"
do
	case "$state" in
	"opt")
		if [ "$arg" = "-d" ]; then
			data="$arg";
		else
			state="$arg"
		fi
		continue;
		;;
	"-k")
		kernel="$arg"
		;;
	"-r")
		rootfs="$arg"
		;;
	"-o")
		output="$arg"
		;;
	"-Q")
		image_seq="$arg"
		;;
	*)
		echo "Invalid option: $arg"
		exit 1
		;;
	esac
	state="opt"
done

ROOTFS_SIG=`hexdump -n 4 -e '"%_p"' "$rootfs"`
[ "$ROOTFS_SIG" = "hsqs" ] || { echo "rootfs is not squashfs."; exit 1; }

KERNEL_SIG=`hexdump -n 4 -e '1/1 "%02x"' "$kernel"`
[ "$KERNEL_SIG" = "d00dfeed" ] || { echo "invalid kernel img"; exit 1; }

UBICFG=`mktemp /tmp/r3600-ubicfg-$image_seq.XXXXXX`

cat <<CFGEND > $UBICFG
[kernel]
mode=ubi
image=$kernel
vol_id=0
vol_type=dynamic
vol_name=kernel
[rootfs]
mode=ubi
image=$rootfs
vol_id=1
vol_type=dynamic
vol_name=ubi_rootfs
CFGEND

[ "$data" = "-d" ] && cat <<CFGEND2 >> $UBICFG
[data]
mode=ubi
vol_size=1
vol_id=2
vol_type=dynamic
vol_name=rootfs_data
vol_flags=autoresize
CFGEND2

[ -z $image_seq ] && \
{ ubinize -m 2048 -p 128KiB -o "$output" "$UBICFG" >/dev/null; } || \
{ ubinize -Q $image_seq -m 2048 -p 128KiB -o "$output" "$UBICFG" >/dev/null; }
